!Este programa solo permite cambiar de quiralidad de manera 
!horizontal y vertical, SIN rotaciones de 180°
program tesis_canonico
implicit none

!**********	DEFINICION DE VARIABLES		*********
!**********	PARAMETROS DEL SISTEMA		*********
!Estos parametros se pueden dejar en otro archivo de lectura
!Los valores de lambda tambien podrian dejarse en otro archivo de lectura
integer, parameter :: side = 50, nsample = side**2, cycles = side**4, nmol = side**2
real(8), parameter :: l1 = 0.9, l2 = 0.45, temperature = 0.1
! En caso de que se quiera modificar o no, pongo el rango de interaccion aparte
real(8) :: lambda

integer :: nps, neigh_number

!**********	MATRICES A USAR			*********
real(8), dimension(nmol, 2) :: mol_centers
real(8), dimension(nmol, 3, 2) :: positions
integer, dimension(nmol) :: chirality, orientation
integer, allocatable, dimension(:, :) :: neighbors
real(8), allocatable, dimension(:, :, :) :: neighbor_pos
real(8), dimension(cycles/nsample + 2) :: system_energy

!**********	VARIABLES INTERNAS		*********
integer :: ii, jj, kk, sample_counter, counter, lambda_iter, size_seed
real(8) :: energy_change, energy, size_sys, eps, zero, pi
integer :: new_chirality
real(8), dimension(3, 2) :: new_pos
character(70) :: chir_format, filename, filename2
character(40) ::  directory
real(8), dimension (4, 2) :: cell
real(8), dimension(32) :: relative_dist
real(8), dimension(2) :: temp_center
real(8), dimension(20) :: unique_dist !De aqui se obtienen las lambda
integer, dimension(8) :: values
integer, dimension(:), allocatable :: seed


!Directorio donde se guardaran los resultados
!El numero de caracteres esta contado: 17
directory = "quiralidad_rotacion_eps/"
print*, directory 

!eps es el valor que le sumaba a lambda para evitar errores de redondeo
!pero parece que no es necesario
eps = 0.0

!Formato de la matriz dependiente del tamaño
!side sustituye a I2
!Es para la escritura de la matriz del sistema
write(chir_format, '( "(", I2, "I4)" )') side

!Calculo de los valores posibles de lambda tal que alcancen todos los sitios
!de interacción de las moleculas primeras vecinas

!Estas son las posible posiciones de los sitios de interacción de una molecula
!Los asteriscos son los sitios de interacción
!      2  *        *  4     
!	  |
!	  |
!	  |
!    l1	  |
!	  |
!	  |
!	  |  
!      1  *--------*  3
!     (0,0)   l2

zero = 0.0 !Es tipo real(8)
 cell(1,:) = (/zero, zero/)
 cell(2,:) = (/zero, l1/)
 cell(3,:) = (/l2,  zero/)
 cell(4,:) = (/l2,  l1/)

!Las posiciones de los sitios de interacción de las moleculas mas cercanas
!se encuentran desplazadas en los ejes x, y 
!Aqui los asteriscos son los centros de red donde se localizan las moleculas vecinas.
!En cada una esta centrada una molecula como la de arriba.
!		*	*	*
!		(-1,1)	(1,0)	(1,1)


!		*	*	*
!		(-1,0)	(0,0)	(1,0)


!		*	*	*
!		(-1,-1)	(-1,0)	(-1,1)

!Con la información de arriba, se calculan las 8*4 = 32 posibles distancias entre
!sitios de interacción de moleculas vecinas
counter = 1
do jj = -1, 1	!Desplazamiento en el eje y
	do ii = -1, 1	!Desplazamiento en el eje x
		do kk = 1, 4	!Numero de sitios de interacción por molecula
			if (ii == 0 .and. jj == 0) then !La molecula central no se incluye
				cycle
			end if
			temp_center = (/dble(ii), dble(jj)/)
			relative_dist(counter) = norm(cell(kk, :) + temp_center)
			counter = counter + 1
		end do
	end do
end do

!write(*,*) "Distancias"
!write(*, "(F20.18)") relative_dist

!Como hay distancias repetidas, hay que ver cuales son unicas
!Hay 20 distancias unicas
unique_dist = 0.0
counter = 1
do ii = 1, 32
	if (.not.(any(abs(unique_dist - relative_dist(ii)) < 0.00001))) then
!	if (.not.(any(unique_dist == relative_dist(ii)))) then
		unique_dist(counter) = relative_dist(ii)
		counter = counter + 1
	end if 
end do

!Cambiar semilla del PRNG con la fecha y hora del momento de ejecución
!A ver si asi cambia la salida 
call date_and_time(VALUES=values)
call random_seed(size=size_seed)
allocate(seed(size_seed))
seed(:) = values(:)
call random_seed(put=seed)


!write(*,*) "Distancias unicas"
!write(*, "(F20.18)") unique_dist

!Se realiza una simulación por cada posible distancia entre sitios de interacción
!Los valores de lambda son precisamente las posibles distancias
do lambda_iter = 1, 20

	lambda = unique_dist(lambda_iter) + eps

	!Numero de vecinos efectivos por "lado"
	!Es para reducir la busqueda de vecinos
	!nps: neighbors per side
	if (lambda <= 1.005) then
		nps = 1
	else
		nps = 2
	end if
	!si nps = 1, entonces neigh_number = 8
!	*  *  *
!	*  0  *
!	*  *  *

	!si nps = 2, entonces neigh_number = 24
!	*  *  *  *  *
!	*  *  *  *  *
!	*  *  0  *  *
!	*  *  *  *  *
!	*  *  *  *  *

	neigh_number = (2*nps + 1)**2 - 1

	!Asignación de memoria despues de calcular neigh_number
	allocate(neighbors(nmol, neigh_number))
	allocate(neighbor_pos(neigh_number + 1, 3, 2))

	!Inicialización
	call create_molecules(nmol, mol_centers, positions, chirality, orientation, neighbors)
	if (lambda_iter == 1) then
		write(*, chir_format) chirality*orientation
	end if

	!Energia inicial
	energy = 0.0
	do jj = 1, nmol
		! Posición de la molecula "central"
		neighbor_pos(1, :, :) = positions(jj, :, :) 
		! Se buscan las posiciones de las vecinas
		do ii = 1, neigh_number
			neighbor_pos(ii + 1, :, :) = positions(neighbors(jj, ii), :, :)
		end do
		energy = energy + potential_energy(neighbor_pos)
	end do
	energy = energy / 2.0
	size_sys = dble(side*side)
	print*, "Energia de ", lambda, " :", energy/size_sys
	
	!Deasignación de tamaño, para asignarle otra dependiendo de nps
	deallocate(neighbors)
	deallocate(neighbor_pos)
end do
!write(*,*) "Energia inicial: ", system_energy(1)
!write(*,*) "Energia final: ", system_energy(nsample**3)

contains

!**********	FUNCIONES Y SUBRUTINAS		*****************

!Norma de un vector
!En el argumento iria la resta de dos vectores si es necesario
real(8) function norm(vr)
	implicit none

	real(8), dimension(2) :: vr
	norm = sqrt(vr(1)**2 + vr(2)**2)

end function norm

!Elegir un numero entero entre a y b, incluidos a y b
integer function rand_int(a, b)
	implicit none
	integer, intent(in) :: a, b ! a < b
	integer :: diff
	real(8) :: rand
	
	diff = dble(b + 1 - a)
	call RANDOM_NUMBER(rand)
	rand_int = floor(diff*rand) + a !El + 1.0 es para que tambien seleccione b

end function rand_int

!Pozo cuadrado
!El argumento es un vector
real(8) function square_well(r_ij)
	implicit none

	real(8), dimension(2) :: r_ij
	real(8) :: ans, dist
	dist = norm(r_ij)
	if (dist == 0.0) then
	    ans = 1000.0	!Es para simular infinitos, tal vez me traiga problemitas
	else if (dist > 0 .and. dist <= lambda) then
	    ans = -1.0
	else !dist > lambda
	    ans = 0.0
	end if
	square_well = ans
end function square_well
	

!Moleculas derechas
!Quiero que la función me regrese una matriz
!real(8) function dextro_molecule(square_pos) NO FUNCIONA
!real(8), dimension(3, 2) function dextro_molecule(square_pos) NO FUNCIONA
function dextro_molecule(square_pos, orientation)
!	Regresa una lista con las posiciones de los atomos de cada molecula para que sea derecha
	implicit none
	real(8), dimension(3, 2) :: pos, dextro_molecule  !Defini el tipo de la función ADENTRO, no antes del nombre
	real(8), dimension(2) :: square_pos
	integer :: orientation, ii
					!  		3
	pos(1, 1) = -l2/2.0  		!		*
	pos(1, 2) = -l1/2.0  		!   		|
	pos(2, 1) = l2/2.0  		!   		|
	pos(2, 2) = -l1/2.0  		!   		|
	pos(3, 1) = l2/2.0  		!   		|
	pos(3, 2) = l1/2.0  		!	*-------*	
					!   	1	2

	do ii = 1, 3
		if (orientation == 1) then
			pos(ii, :) = square_pos(:) + pos(ii, :)
		else if (orientation == 2) then
			! El signo menos basta para voltearla 180 grados
			pos(ii, :) = square_pos(:) - pos(ii, :)
		else
			print*, "ERROR DERECHAS"
		end if
	end do
	!La siguiente linea era lo que me faltaba :v
	dextro_molecule = pos
end function dextro_molecule

!Moleculas izquierdas
function levo_molecule(square_pos, orientation)
!	Regresa una lista con las posiciones de los atomos de cada molecula para que sea derecha
	implicit none
	real(8), dimension(3, 2) :: pos, levo_molecule  !Defini el tipo de la función ADENTRO, no antes del nombre
	real(8), dimension(2) :: square_pos
	integer :: orientation, ii
					!   	3
	pos(1, 1) = l2/2.0 		!	*
	pos(1, 2) = -l1/2.0  		!	|
	pos(2, 1) = -l2/2.0  		!	|
	pos(2, 2) = -l1/2.0  		!	|
	pos(3, 1) = -l2/2.0  		!	|
	pos(3, 2) = l1/2.0  		!	*-------*
					!	2	1
	do ii = 1, 3
		if (orientation == 1) then
			pos(ii, :) = square_pos(:) + pos(ii, :)
		else if (orientation == 2) then
			pos(ii, :) = square_pos(:) - pos(ii, :)
		else
			print*, "ERROR IZQUIERDAS"
		end if
	end do

	levo_molecule = pos
end function levo_molecule

subroutine create_molecules(nmol, mol_centers, positions, chirality, orientation, neighbors)
	implicit none
	integer, intent(in) :: nmol
	real(8), dimension(nmol, 2), intent(out) :: mol_centers
	real(8), dimension(nmol, 3, 2), intent(out) :: positions
	integer, dimension(nmol), intent(out) :: chirality, orientation
	integer, dimension(nmol, neigh_number), intent(out) :: neighbors
	real(8) :: rand
	integer :: ii, jj, row, col, mol_index, counter
	
	! Hacer que "la mitad" de las moleculas tengan la misma quiralidad de manera aleatoria
	do ii = 1, nmol
		!call RANDOM_NUMBER(rand)
		!if (rand < 0.5) then
			chirality(ii) = 1 !"D"
		!else
		!	chirality(ii) = -1 !"L"
		!end if

		! Creo que es mejor asi y no con rand_int para decir cual
		! orientación es cual
		!call RANDOM_NUMBER(rand)
		!Generar bandas verticales
		if (mod(floor(dble(ii - 1)/dble(side)), 2) == 0) then
			if (mod(ii, 2) == 0) then 
			orientation(ii) = 1 !Lado corto abajo
		else
			orientation(ii) = 2 !Lado corto arriba
		end if
		
		else
			if (mod(ii, 2) == 0) then 
			orientation(ii) = 2 !Lado corto abajo
		else
			orientation(ii) = 1 !Lado corto arriba
			end if
		end if
			
	end do

	!chirality = 1
	    
	! Situa las moleculas en posiciones y orientaciones aleatorias
	do jj = 0, (side - 1)
		do ii = 0, (side - 1)
			mol_index = jj*side + ii + 1
			mol_centers(mol_index, 1) = ii
			mol_centers(mol_index, 2) = jj
			if (chirality(mol_index) == -1) then
				!temp_pos = levo_molecule(mol_center(mol_index, :))
				positions(mol_index, :, :) = \&
				levo_molecule(mol_centers(mol_index, :), orientation(mol_index))
			else if (chirality(mol_index) == 1) then
			!temp_pos = dextro_molecule(mol_center(mol_index, :))
				positions(mol_index, :, :) = \&
				dextro_molecule(mol_centers(mol_index, :), orientation(mol_index))
			end if
			!Asignar vecinos
			!nps esta definido desde arriba
			counter = 1
			do col = -nps, nps
				do row = -nps, nps
					if (col == 0 .and. row == 0) then 
						cycle 	!Se salta la molecula central porque no es su propia vecina
					else
						neighbors(mol_index, counter) = modulo(jj + col, side)*side + modulo(ii + row, side) + 1 
						counter = counter + 1
					end if
				end do
			end do
		end do
	end do
end subroutine create_molecules


real(8) function potential_energy(positions)
	implicit none
	! positions incluye las posiciones de cada atomo
	! positions[1] es la molecula central

	real(8), dimension(neigh_number + 1, 3, 2) :: positions
	integer :: ii, jj, kk, aa
	real(8) :: energy, temp_dist
	real(8), dimension(2) :: dist

	energy = 0.0
	do kk = 2, (neigh_number + 1)	!Iterar sobre las particulas vecinas
		do jj = 1, 3	!	Sobre los sitios atractivos de las vecinas
			do ii = 1, 3	!Sobre las de la particula central
				! Sin condiciones periódicas
				!energy += square_well(positions[1][ii] - positions[kk][jj])
		    
			    	!Con condiciones periódicas
				do aa = 1, 2    !Coordenadas de los sitios 
					temp_dist = abs(positions(1, ii, aa) - positions(kk, jj, aa))
					if (temp_dist > side/2) then
						temp_dist = side - temp_dist
					end if
					dist(aa) = temp_dist
				end do
				energy = energy + square_well(dist)
			end do
		end do
	end do
	potential_energy = energy
end function potential_energy

end program tesis_canonico
