#Este programa cuenta el tamaño y número de cúmulos, además del exceso enantiomérico

function count_cluster(system)
    nmols = length(system)
    is_bond = fill(false, nmols)
    is_checked = fill(false, nmols)
    chirality = system
    neighbors = Vector(nmols)
    cluster = zeros(Int64, nmols)
    
    
    side = floor(Int64, sqrt(nmols))
    for jj in 0:(side - 1)
        for ii in 0:(side - 1)
            mol_index = jj*side + ii + 1
    
            upper = mod(jj + 1, side)*side + ii + 1
            lower = mod(jj - 1, side)*side + ii + 1
            left  = jj*side + mod(ii - 1, side) + 1
            right = jj*side + mod(ii + 1, side) + 1 
            neighbors[mol_index] = [upper, lower, left, right]
        end
    end
    
    #Crear varios cúmulos
    ncluster = 1
    while sum(is_checked) < nmols
        idx = 0
        for ii in 1:nmols
            if cluster[ii] == 0 && !is_checked[ii] #if !is\_checked
                idx = ii
                cluster[idx] = ncluster
                break
            end
        end
# Crear un cúmulo
    # Si todos los sitios "unidos" ya han sido revisados, entonces ya se formó un cúmulo
    # Siempre habrá menor o igual número de sitios revisados que de unidos, por lo que si 
    # no se cumple la condición, ya se revisaron todos los sitios
        while is_checked[idx] == false
            for nb in neighbors[idx]
                # Intentar unir dos sitios vecinos
                if (is_checked[nb] == false && chirality[nb] == chirality[idx])
                     is_bond[idx] = true
                     is_bond[nb] = true
                     cluster[nb] = ncluster
                end
            end
            is_checked[idx] = true
        # Escoger uno de los sitios del cúmulo para revisar sus vecinos
            for ii in 1:nmols 
                if cluster[ii] == ncluster && !is_checked[ii]
                    idx = ii
                    break
                end
            end
        end
        
        ncluster = ncluster + 1
    end
    return cluster
end

direc_arriba = pwd()*"/lado50_tiempo5_enfr/"
directs = readdir(direc_arriba);

for direct in directs
println("Directorio actual: "*direct)
direc = direc_arriba*direct*"/"
files = readdir(direc)

cluster_sizes = Array(Int64, (20, 2))
mol_count = Array(Float64, 20)
count_file = 1

for (ii, file) in enumerate(files)
    words = ["inicial", "final"]
    is_word = map((x) -> contains(file, x), words)
    forbidden_words = ["levos", "dextros", "energies", "histograma", "grafica", "param", "exceso", "tamano"]
    is_forbidden_word = map((x) -> contains(file, x), forbidden_words)

     # Escoger archivos con palabras "especiales" para graficar
    if true in is_word && !(true in is_forbidden_word)
		try
		    # Graficar sistemas
			sys_data = sign(readdlm(direc*file)) #Con el sign escoge por quiralidad
			nmols = length(sys_data)
		    side = floor(Int64, sqrt(nmols))
		    # Contar clusters, guardar datos (que podría ir en el programa de la simulación) y graficar
		    clusters = count_cluster(sys_data)
		    cluster_list = []
		    for nclust in 1:maximum(clusters)
		        sc = []
		        for ii in 1:length(clusters)
		            if clusters[ii] == nclust
		                append!(sc, ii)
		            end
		        end
		        append!(cluster_list, [sc])
		    end
		    size_of_clusters = sort(map(length, cluster_list), rev=true)#/nmols
			writedlm(direc*"datos_histograma_"*file, size_of_clusters)

			histogram = zeros(Int64, nmols, 2)
			order_parameter = zeros(Float64, nmols, 2)
			for jj in 1:nmols
				histogram[jj, 1] = jj
				for size_clust in size_of_clusters
					if size_clust == jj
						histogram[jj, 2] = histogram[jj, 2] + 1
					end
				end
			end
			writedlm(direc*"histograma_"*file, histogram)



			# A partir de aquí, veré cuántos clusters hay de mas del 10\% del tamaño del sistema 
			medium_clusters = count(size_clust -> (size_clust > 0.05), size_of_clusters/nmols)
			big_clusters = count(size_clust -> (size_clust > 0.1), size_of_clusters/nmols)
			cluster_sizes[count_file, :] = [medium_clusters, big_clusters]
			writedlm(direc*"tamano_clusters"*string(side)*".txt", cluster_sizes)

			#Calcular exceso enantiomérico
			der = count(chir -> (chir == 1), sys_data)
			izq = nmols - der
			ee = abs(der - izq)/Int64(nmols)
			mol_count[count_file] = ee

			count_file = count_file + 1
		catch err
			println(err)
		end
	end
end
writedlm(direc*"exceso_en.txt", mol_count)
end

Vector
