!Este programa solo permite cambiar de quiralidad de manera 
!horizontal y vertical, SIN rotaciones de 180°

!Se sustituyó el guion bajo con diagonal-guion bajo para que salgan bien los comentarios

program tesis_canonico
implicit none

!**********	DEFINICION DE VARIABLES		*********
!**********	PARAMETROS DEL SISTEMA		*********
!Estos parámetros se pueden dejar en otro archivo de lectura
!Los valores de lambda también podrían dejarse en otro archivo de lectura
integer, parameter :: side = 50, nsample = side**2, cycles = side**5, nmol = side**2, ntemps = side
real(8), parameter :: l1 = 0.9, l2 = 0.45, init_temperature = 3.0, final_temperature = 0.3
! En caso de que se quiera modificar o no, pongo el rango de interaccion aparte
real(8) :: lambda

integer :: nps, neigh_number
integer :: i_lambda = 1, f_lambda = 20

!**********	MATRICES A USAR			*********
real(8), dimension(nmol, 2) :: mol_centers
real(8), dimension(nmol, 3, 2) :: positions
integer, dimension(nmol) :: chirality, orientation
integer, allocatable, dimension(:, :) :: neighbors
real(8), allocatable, dimension(:, :, :) :: neighbor_pos
real(8), dimension(cycles/nsample + 2) :: system_energy

!**********	VARIABLES INTERNAS		*********
integer :: ii, jj, kk, sample_counter, counter, lambda_iter, size_seed, cycles_ptemp, temp_counter
real(8) :: energy_change, energy, size_sys, eps, zero, pi, temperature, dtemp, quad_a, quad_b
real(8), dimension(ntemps + 1) :: temp_arr
integer :: new_chirality
real(8), dimension(3, 2) :: new_pos
character(1000) :: chir_format, filename, filename2
character(1000) ::  directory
real(8), dimension (4, 2) :: cell
real(8), dimension(32) :: relative_dist
real(8), dimension(2) :: temp_center
real(8), dimension(20) :: unique_dist !De aquí se obtienen las lambda
integer, dimension(8) :: values
integer, dimension(:), allocatable :: seed


!Directorio donde se guardarán los resultados
directory = "lado50_tiempo5_enfr/temp03/"
print*, trim(directory)

!eps es el valor que le sumaba a lambda para evitar errores de redondeo
!pero parece que no es necesario
eps = 0.0

!Formato de la matriz dependiente del tamaño
!side sustituye a I2
!Es para la escritura de la matriz del sistema
write(chir_format, '( "(", I2, "I4)" )') side

!Cálculo de los valores posibles de lambda tal que alcancen todos los sitios
!de interacción de las moléculas primeras vecinas

!Estas son las posible posiciones de los sitios de interacción de una molécula
!Los asteriscos son los sitios de interacción
!      2  *        *  4     
!	  |
!	  |
!	  |
!    l1	  |
!	  |
!	  |
!	  |  
!      1  *--------*  3
!     (0,0)   l2

zero = 0.0 !Es tipo real(8)
 cell(1,:) = (/zero, zero/)
 cell(2,:) = (/zero, l1/)
 cell(3,:) = (/l2,  zero/)
 cell(4,:) = (/l2,  l1/)

!Las posiciones de los sitios de interacción de las moléculas más cercanas
!se encuentran desplazadas en los ejes x, y 
!Aquí los asteriscos son los centros de red donde se localizan las moléculas vecinas.
!En cada una está centrada una molécula como la de arriba.
!		*	*	*
!		(-1,1)	(1,0)	(1,1)


!		*	*	*
!		(-1,0)	(0,0)	(1,0)


!		*	*	*
!		(-1,-1)	(-1,0)	(-1,1)

!Con la información de arriba, se calculan las 8*4 = 32 posibles distancias entre
!sitios de interacción de moléculas vecinas
counter = 1
do jj = -1, 1	!Desplazamiento en el eje y
	do ii = -1, 1	!Desplazamiento en el eje x
		do kk = 1, 4	!Número de sitios de interacción por molécula
			if (ii == 0 .and. jj == 0) then !La molécula central no se incluye
				cycle
			end if
			temp_center = (/dble(ii), dble(jj)/)
			relative_dist(counter) = norm(cell(kk, :) + temp_center)
			counter = counter + 1
		end do
	end do
end do

!write(*,*) "Distancias"
!write(*, "(F20.18)") relative_dist

!Como hay distancias repetidas, hay que ver cuáles son únicas
!Hay 20 distancias únicas
unique_dist = 0.0
counter = 1
do ii = 1, 32
	if (.not.(any(abs(unique_dist - relative_dist(ii)) < 0.00001))) then
!	if (.not.(any(unique_dist == relative_dist(ii)))) then
		unique_dist(counter) = relative_dist(ii)
		counter = counter + 1
	end if 
end do

!Cambiar semilla del PRNG con la fecha y hora del momento de ejecución
!A ver si así cambia la salida 
call date_and_time(VALUES=values)
call random_seed(size=size_seed)
allocate(seed(size_seed))
seed(:) = values(:)
call random_seed(put=seed)


!write(*,*) "Distancias únicas"
!write(*, "(F20.18)") unique_dist

!Se realiza una simulación por cada posible distancia entre sitios de interacción
!Los valores de lambda son precisamente las posibles distancias

do lambda_iter = i_lambda, f_lambda
	lambda = unique_dist(lambda_iter) + eps
	write(*,*) "lambda = ", lambda

	!Número de vecinos efectivos por "lado"
	!Es para reducir la búsqueda de vecinos
	!nps: neighbors per side
	if (lambda <= 1.005) then
		nps = 1
	else
		nps = 2
	end if
	!si nps = 1, entonces neigh_number = 8
!	*  *  *
!	*  0  *
!	*  *  *

	!si nps = 2, entonces neigh_number = 24
!	*  *  *  *  *
!	*  *  *  *  *
!	*  *  0  *  *
!	*  *  *  *  *
!	*  *  *  *  *

	neigh_number = (2*nps + 1)**2 - 1

	!Asignación de memoria después de calcular neigh_number
	allocate(neighbors(nmol, neigh_number))
	allocate(neighbor_pos(neigh_number + 1, 3, 2))

	!Inicialización
	call create_molecules(nmol, mol_centers, positions, chirality, orientation, neighbors)


	!Energía inicial
	energy = 0.0
	do jj = 1, nmol
		! Posición de la molécula "central"
		neighbor_pos(1, :, :) = positions(jj, :, :) 
		! Se buscan las posiciones de las vecinas
		do ii = 1, neigh_number
			neighbor_pos(ii + 1, :, :) = positions(neighbors(jj, ii), :, :)
		end do
		energy = energy + potential_energy(neighbor_pos)
	end do
	energy = energy / 2.0
	size_sys = dble(side*side)
	system_energy(1) = energy/size_sys

	! Ciclo de Montecarlo
	! Se modificó para cambiar la temperatura poco a poco
	dtemp = (init_temperature - final_temperature)/(ntemps - 1)

	!Quiero convertir el dt lineal a uno cuadrático, para que la temperatura descienda rapido
	!al principio y lento al final, a ver si funciona para correr con side**4	
	quad_a = (init_temperature - final_temperature)/(init_temperature**2 - final_temperature**2)  
	quad_b = final_temperature - quad_a*final_temperature**2
	temp_arr(1) = init_temperature
	do ii = 1, (ntemps - 2)
		temp_arr(ii + 1) = quad_a*(init_temperature - ii*dtemp)**2 + quad_b
	end do
	temp_arr(ntemps) = final_temperature
	temp_arr(ntemps + 1) = final_temperature

	write(*, *) temp_arr
	temperature = init_temperature
	!Número de ciclos que se hacen por cada temperatura
	cycles_ptemp = floor(dble(cycles)/ntemps)
	
	sample_counter = 1
	temp_counter = 1
	do ii = 1, cycles
		call montecarlo(nmol, mol_centers, positions, chirality, orientation, &
			neighbors, temperature, energy_change)
		energy = energy + energy_change
		!Muestreo
		if (mod(ii, nsample) == 0) then
			system_energy(sample_counter) = energy/size_sys
			sample_counter = sample_counter + 1
			!if (ii == cycles/10) then
			!	write(*,*) "Avance: ", 100*ii/cycles, "%"
			!end if
		end if
		!Bajar la temperatura cada cierto número de ciclos
		if (mod(ii, cycles_ptemp) == 0) then
!			print*, "Temp. intermedia: ", temperature
			temp_counter = temp_counter + 1
!			print*, temp_counter
			temperature = temp_arr(temp_counter)
		end if

	end do

	! Escritura de archivos
	! La siguiente línea es para variar el nombre del archivo conforme a lambda
	! Archivo de la configuración del sistema
	! Aquí es donde se debería multiplicar chirality*orientation para 
	! tener en un solo número la quiralidad y la orientación
	write(filename, '( A, "lambda", F7.5, "_final.txt" )') trim(directory), lambda
	print*, trim(filename)
	open (10, file = filename, status='UNKNOWN')
	write(10, chir_format) chirality*orientation
	close(10)
	
	!Archivo de las energías
	write(filename2, '( A, "t_energies_lambda", F7.5, ".txt" )') trim(directory), lambda
	open(20, file = filename2, status='UNKNOWN')
	write(20, *) system_energy
	close(20)

	!Deasignación de tamaño, para asignarle otra dependiendo de nps
	deallocate(neighbors)
	deallocate(neighbor_pos)
end do

contains

!**********	FUNCIONES Y SUBRUTINAS		*****************

!Norma de un vector
!En el argumento iría la resta de dos vectores si es necesario
real(8) function norm(vr)
	implicit none

	real(8), dimension(2) :: vr
	norm = sqrt(vr(1)**2 + vr(2)**2)

end function norm

!Elegir un número entero entre a y b, incluidos a y b
integer function rand_int(a, b)
	implicit none
	integer, intent(in) :: a, b ! a < b
	integer :: diff
	real(8) :: rand
	
	diff = dble(b + 1 - a)
	call RANDOM_NUMBER(rand)
	rand_int = floor(diff*rand) + a !El + 1.0 es para que tambien seleccione b

end function rand_int

!Pozo cuadrado
!El argumento es un vector
real(8) function square_well(r_ij)
	implicit none

	real(8), dimension(2) :: r_ij
	real(8) :: ans, dist
	dist = norm(r_ij)
	if (dist == 0.0) then
	    ans = 1000.0	!Es para simular infinitos, tal vez me traiga problemitas
	else if (dist > 0 .and. dist <= lambda) then
	    ans = -1.0
	else !dist > lambda
	    ans = 0.0
	end if
	square_well = ans
end function square_well
	

!Moléculas derechas
!Quiero que la función me regrese una matriz
!real(8) function dextro_molecule(square_pos) NO FUNCIONA
!real(8), dimension(3, 2) function dextro_molecule(square_pos) NO FUNCIONA
function dextro_molecule(square_pos, orientation)
!	Regresa una lista con las posiciones de los átomos de cada molécula para que sea derecha
	implicit none
	real(8), dimension(3, 2) :: pos, dextro_molecule  !Definí el tipo de la función ADENTRO, no antes del nombre
	real(8), dimension(2) :: square_pos
	integer :: orientation, ii
					!  		3
	pos(1, 1) = -l2/2.0  		!		*
	pos(1, 2) = -l1/2.0  		!   		|
	pos(2, 1) = l2/2.0  		!   		|
	pos(2, 2) = -l1/2.0  		!   		|
	pos(3, 1) = l2/2.0  		!   		|
	pos(3, 2) = l1/2.0  		!	*-------*	
					!   	1	2

	do ii = 1, 3
		if (orientation == 1) then
			pos(ii, :) = square_pos(:) + pos(ii, :)
		else if (orientation == 2) then
			! El signo menos basta para voltearla 180 grados
			pos(ii, :) = square_pos(:) - pos(ii, :)
		else
			print*, "ERROR DERECHAS"
		end if
	end do
	!La siguiente línea era lo que me faltaba :v
	dextro_molecule = pos
end function dextro_molecule

!Moléculas izquierdas
function levo_molecule(square_pos, orientation)
!	Regresa una lista con las posiciones de los átomos de cada molécula para que sea derecha
	implicit none
	real(8), dimension(3, 2) :: pos, levo_molecule  !Definí el tipo de la función ADENTRO, no antes del nombre
	real(8), dimension(2) :: square_pos
	integer :: orientation, ii
					!   	3
	pos(1, 1) = l2/2.0 		!	*
	pos(1, 2) = -l1/2.0  		!	|
	pos(2, 1) = -l2/2.0  		!	|
	pos(2, 2) = -l1/2.0  		!	|
	pos(3, 1) = -l2/2.0  		!	|
	pos(3, 2) = l1/2.0  		!	*-------*
					!	2	1
	do ii = 1, 3
		if (orientation == 1) then
			pos(ii, :) = square_pos(:) + pos(ii, :)
		else if (orientation == 2) then
			pos(ii, :) = square_pos(:) - pos(ii, :)
		else
			print*, "ERROR IZQUIERDAS"
		end if
	end do

	levo_molecule = pos
end function levo_molecule

subroutine create_molecules(nmol, mol_centers, positions, chirality, orientation, neighbors)
	implicit none
	integer, intent(in) :: nmol
	real(8), dimension(nmol, 2), intent(out) :: mol_centers
	real(8), dimension(nmol, 3, 2), intent(out) :: positions
	integer, dimension(nmol), intent(out) :: chirality, orientation
	integer, dimension(nmol, neigh_number), intent(out) :: neighbors
	real(8) :: rand
	integer :: ii, jj, row, col, mol_index, counter
	
	! Hacer que "la mitad" de las moléculas tengan la misma quiralidad de manera aleatoria
	do ii = 1, nmol
		call RANDOM_NUMBER(rand)
		if (rand < 0.5) then
			chirality(ii) = 1 !"D"
		else
			chirality(ii) = -1 !"L"
		end if

		! Creo que es mejor así y no con rand_int para decir cuál
		! orientación es cuál
		call RANDOM_NUMBER(rand)
		if (rand < 0.5) then
			orientation(ii) = 1 !Lado corto abajo
		else
			orientation(ii) = 2 !Lado corto arriba
		end if
	end do

	!chirality = 1
	    
	! Sitúa las moléculas en posiciones y orientaciones aleatorias
	do jj = 0, (side - 1)
		do ii = 0, (side - 1)
			mol_index = jj*side + ii + 1
			mol_centers(mol_index, 1) = ii
			mol_centers(mol_index, 2) = jj
			if (chirality(mol_index) == -1) then
				!temp_pos = levo_molecule(mol_center(mol_index, :))
				positions(mol_index, :, :) = &
				levo_molecule(mol_centers(mol_index, :), orientation(mol_index))
			else if (chirality(mol_index) == 1) then
			!temp_pos = dextro_molecule(mol_center(mol_index, :))
				positions(mol_index, :, :) = &
				dextro_molecule(mol_centers(mol_index, :), orientation(mol_index))
			end if
			!Asignar vecinos
			!nps está definido desde arriba
			counter = 1
			do col = -nps, nps
				do row = -nps, nps
					if (col == 0 .and. row == 0) then 
						cycle 	!Se salta la molécula central porque no es su propia vecina
					else
						neighbors(mol_index, counter) = modulo(jj + col, side)*side + modulo(ii + row, side) + 1 
						counter = counter + 1
					end if
				end do
			end do
		end do
	end do
end subroutine create_molecules

! Esta función selecciona la quiralidad contraria y elige al azar una orientación, 
! que es equivalente a elegir al azar voltear horizontal o verticalmente
! Para 4 posibles rotaciones tal vez deba cambiar el programa
subroutine change_chirality(mol_chirality, center, new_chirality, new_pos, new_orientation)
implicit none
integer, intent(in) :: mol_chirality
real(8), dimension(2), intent(in) :: center
integer, intent(out) :: new_chirality, new_orientation
real(8), dimension(3, 2), intent(out) :: new_pos
real(8) :: rand

	! Se escoge al azar si se voltea sobre
	! el lado largo o el corto
	new_orientation = rand_int(1, 2)
	
	!Si es derecha, cambiar a izquierda
	if (mol_chirality == -1) then
		new_pos = dextro_molecule(center, new_orientation)
	else if (mol_chirality == 1) then
		new_pos = levo_molecule(center, new_orientation)
	end if
	new_chirality = -mol_chirality

end subroutine change_chirality

!Esta función rota 180 grados una particula, sin cambiar su quiralidad
subroutine rotate_molecule(mol_chirality, center, old_orientation, new_pos, new_orientation)
	integer, intent(in) :: mol_chirality, old_orientation
	real(8), dimension(2), intent(in) :: center
	real(8), dimension(3, 2), intent(out) :: new_pos
	integer, intent(out) :: new_orientation
	
	!Conserva quiralidad, solo rota	
	if (mol_chirality == 1) then
		if (old_orientation == 1) then
			new_orientation = 2
			new_pos = dextro_molecule(center, new_orientation)
		else if (old_orientation == 2) then
			new_orientation = 1
			new_pos = dextro_molecule(center, new_orientation)
		end if
	else if (mol_chirality == -1) then
		if (old_orientation == 1) then
			new_orientation = 2
			new_pos = levo_molecule(center, new_orientation)
		else if (old_orientation == 2) then
			new_orientation = 1
			new_pos = levo_molecule(center, new_orientation)
		end if
	end if
end subroutine rotate_molecule

real(8) function potential_energy(positions)
	implicit none
	! positions incluye las posiciones de cada átomo
	! positions[1] es la molécula central

	real(8), dimension(neigh_number + 1, 3, 2) :: positions
	integer :: ii, jj, kk, aa
	real(8) :: energy, temp_dist
	real(8), dimension(2) :: dist

	energy = 0.0
	do kk = 2, (neigh_number + 1)	!Iterar sobre las partículas vecinas
		do jj = 1, 3	!	Sobre los sitios atractivos de las vecinas
			do ii = 1, 3	!Sobre las de la partícula central
				! Sin condiciones periódicas
				!energy += square_well(positions[1][ii] - positions[kk][jj])
		    
			    	!Con condiciones periódicas
				do aa = 1, 2    !Coordenadas de los sitios 
					temp_dist = abs(positions(1, ii, aa) - positions(kk, jj, aa))
					if (temp_dist > side/2) then
						temp_dist = side - temp_dist
					end if
					dist(aa) = temp_dist
				end do
				energy = energy + square_well(dist)
			end do
		end do
	end do
	potential_energy = energy
end function potential_energy

!Criterio de Metropolis
logical function metropolis_step(exp_arg)
!Returns True if the trial configuration goes towards a region with higher probability
!or it is given the chance to explore regions with fewer probability

implicit none

real(8) :: exp_arg, w, rand
logical :: accepted

	if (exp_arg < 0.0) then
		accepted = .true. ! Se actualiza el estado del sistema
	else
		w = exp(-exp_arg)
		call RANDOM_NUMBER(rand)
		if (rand < w) then ! Energía: w
			accepted = .true. ! También se actualiza el estado del sistema
	else
		accepted = .false.
		end if
	end if
	metropolis_step = accepted

end function metropolis_step

subroutine montecarlo(nmol, centers, positions, chirality, orientation, neighbors, t_env, energy_change)
	implicit none
	! Variables de entrada y salida
	integer, intent(in) :: nmol
	real(8), dimension(nmol, 2), intent(in) :: centers
	integer, dimension(nmol, neigh_number), intent(in) :: neighbors
	real(8), dimension(nmol, 3, 2), intent(inout) :: positions
	integer, dimension(nmol), intent(inout) :: chirality, orientation
	real(8), intent(in) :: t_env
	real(8), intent(out) :: energy_change

	!Variables internas
	integer :: ii, idx, neighbor, trial_chir 
	real(8), dimension(neigh_number + 1, 3, 2) :: neighbor_pos, neighbor_pos2
	real(8), dimension(3, 2) :: trial_pos
	real(8) :: old_energy, trial_energy, energy_diff, rand
	integer :: trial_orientation
	logical :: accepted
	
	! Elegir una molécula al azar
	idx = rand_int(1, side*side)
	! Buscar las posiciones de los átomos de esa molécula y de sus vecinas
	neighbor_pos(1, :, :) = positions(idx, :, :)
	!write(*,*) "Posición: ", positions(idx, :, :) 
	do ii = 1, neigh_number
		neighbor = neighbors(idx, ii)
		neighbor_pos(ii + 1, :, :) = positions(neighbor, :, :)
	end do
	
	! Intento de cambio
	! En este caso, puede elegir entre cambiar quiralidad o rotar 
	call random_number(rand)
	if (rand < 0.5) then
		call change_chirality(chirality(idx), centers(idx, :), trial_chir, &
		trial_pos, trial_orientation)
	else
		call rotate_molecule(chirality(idx), centers(idx, :), orientation(idx), &
		trial_pos, trial_orientation)
		trial_chir = chirality(idx)
	end if

	! Buscar las posiciones de los átomos de esa molécula y de sus vecinas
	neighbor_pos2(1, :, :) = trial_pos
	do ii = 1, neigh_number
		neighbor = neighbors(idx, ii)
		neighbor_pos2(ii + 1, :, :) = positions(neighbor, :, :)
	end do	

	! Cálculo de energías
	old_energy = potential_energy(neighbor_pos)
	trial_energy = potential_energy(neighbor_pos2)

	! La diferencia de energías se utiliza para aceptar o rechazar el movimiento. 
	! energy_change entrega la diferencia de energías si el movimiento fue aceptado.
	! Si fue rechazado, no hay cambio de energía en el sistema.
	energy_diff = trial_energy - old_energy
	!write(*,*) "Diferencia de energías: ", energy_diff
	energy_change = 0.0
	accepted = metropolis_step(energy_diff/t_env)
	if (accepted) then
		positions(idx, :, :) = trial_pos
		chirality(idx) = trial_chir
		orientation(idx) = trial_orientation
		energy_change = energy_diff
	end if


end subroutine montecarlo

end program tesis_canonico
