!Este programa solo permite cambiar de quiralidad de manera 
!horizontal y vertical, SIN rotaciones de 180°

!Se sustituyó el guion bajo con diagonal-guion bajo para que salgan bien los comentarios
program tesis_canonico
implicit none

!**********	DEFINICION DE VARIABLES		*********
!**********	PARAMETROS DEL SISTEMA		*********
!Estos parametros se pueden dejar en otro archivo de lectura
!Los valores de lambda tambien podrian dejarse en otro archivo de lectura
integer, parameter :: side = 30, nsample = side**2, cycles = side**5, nsites = side**2, ntemps = side
real(8), parameter :: l1 = 0.9, l2 = 0.45, init_temperature = 3.0, final_temperature = 0.3
real(8), parameter :: chempot = 40.0 !Potencial quimico	
real(8) :: lambdawl !Longitud de onda termica
real(8) :: zz !Actividad
! En caso de que se quiera modificar o no, pongo el rango de interaccion aparte
real(8) :: lambda
integer :: nmol = floor(dble(nsites)/10) !Numero de particulas (inicial, luego cambia)
integer :: nps, neigh_number
integer :: initial_iter = 1, final_iter = 20

!**********	MATRICES A USAR			*********
real(8), dimension(nsites, 2) :: mol_centers
real(8), dimension(nsites, 3, 2) :: positions
integer, dimension(nsites) :: chirality, orientation
logical, dimension(nsites) :: occupancy
integer, allocatable, dimension(:, :) :: neighbors
real(8), allocatable, dimension(:, :, :) :: neighbor_pos
logical, allocatable, dimension(:) :: neighbor_occ
real(8), dimension(cycles/nsample + 2) :: system_energy

!**********	VARIABLES INTERNAS		*********
integer :: ii, jj, kk, sample_counter, counter, lambda_iter, size_seed, cycles_ptemp, temp_counter
real(8) :: energy_change, energy, size_sys, eps, zero, pi, temperature, dtemp, quad_a, quad_b
real(8), dimension(ntemps + 1) :: temp_arr
integer :: new_chirality
real(8), dimension(3, 2) :: new_pos
character(70) :: chir_format, filename, filename2
character(40) ::  directory
real(8), dimension (4, 2) :: cell
real(8), dimension(32) :: relative_dist
real(8), dimension(2) :: temp_center
real(8), dimension(20) :: unique_dist !De aqui se obtienen las lambda
integer, dimension(8) :: values
integer, dimension(:), allocatable :: seed


!Directorio donde se guardaran los resultados
directory = "modif3/"
print*, trim(directory)

!eps es el valor que le sumaba a lambda para evitar errores de redondeo
!pero parece que no es necesario
eps = 0.0

!Formato de la matriz dependiente del tamaño
!side sustituye a I2
!Es para la escritura de la matriz del sistema
write(chir_format, '( "(", I2, "I4)" )') side

!Calculo de los valores posibles de lambda tal que alcancen todos los sitios
!de interacción de las moleculas primeras vecinas

!Estas son las posible posiciones de los sitios de interacción de una molecula
!Los asteriscos son los sitios de interacción
!      2  *        *  4     
!	  |
!	  |
!	  |
!l1	  |
!	  |
!	  |
!	  |  
!  1  *--------*  3
!     (0,0)   l2

zero = 0.0 !Es tipo real(8)
 cell(1,:) = (/zero, zero/)
 cell(2,:) = (/zero, l1/)
 cell(3,:) = (/l2,  zero/)
 cell(4,:) = (/l2,  l1/)

!Las posiciones de los sitios de interacción de las moleculas mas cercanas
!se encuentran desplazadas en los ejes x, y 
!Aqui los asteriscos son los centros de red donde se localizan las moleculas vecinas.
!En cada una esta centrada una molecula como la de arriba.
!		*	*	*
!		(-1,1)	(1,0)	(1,1)


!		*	*	*
!		(-1,0)	(0,0)	(1,0)


!		*	*	*
!		(-1,-1)	(-1,0)	(-1,1)

!Con la información de arriba, se calculan las 8*4 = 32 posibles distancias entre
!sitios de interacción de moleculas vecinas
counter = 1
do jj = -1, 1	!Desplazamiento en el eje y
	do ii = -1, 1	!Desplazamiento en el eje x
		do kk = 1, 4	!Numero de sitios de interacción por molecula
			if (ii == 0 .and. jj == 0) then !La molecula central no se incluye
				cycle
			end if
			temp_center = (/dble(ii), dble(jj)/)
			relative_dist(counter) = norm(cell(kk, :) + temp_center)
			counter = counter + 1
		end do
	end do
end do

!Como hay distancias repetidas, hay que ver cuales son unicas
!Hay 20 distancias unicas
unique_dist = 0.0
counter = 1
do ii = 1, 32
	if (.not.(any(abs(unique_dist - relative_dist(ii)) < 0.00001))) then
!	if (.not.(any(unique_dist == relative_dist(ii)))) then
		unique_dist(counter) = relative_dist(ii)
		counter = counter + 1
	end if 
end do

!Cambiar semilla del PRNG con la fecha y hora del momento de ejecución
!A ver si asi cambia la salida 
call date_and_time(VALUES=values)
call random_seed(size=size_seed)
allocate(seed(size_seed))
seed(:) = values(:)
call random_seed(put=seed)

!Se realiza una simulación por cada posible distancia entre sitios de interacción
!Los valores de lambda son precisamente las posibles distancias
do lambda_iter = initial_iter, final_iter
	lambda = unique_dist(lambda_iter) + eps
	write(*,*) "lambda = ", lambda

	!Numero de vecinos efectivos por "lado"
	!Es para reducir la busqueda de vecinos
	!nps: neighbors per side
	if (lambda <= 1.005) then
		nps = 1
	else
		nps = 2
	end if
	!si nps = 1, entonces neigh_number = 8
!	*  *  *
!	*  0  *
!	*  *  *

	!si nps = 2, entonces neigh_number = 24
!	*  *  *  *  *
!	*  *  *  *  *
!	*  *  0  *  *
!	*  *  *  *  *
!	*  *  *  *  *

	neigh_number = (2*nps + 1)**2 - 1

	!Asignación de memoria despues de calcular neigh_number
	allocate(neighbors(nsites, neigh_number))
	allocate(neighbor_pos(neigh_number + 1, 3, 2))
	allocate(neighbor_occ(neigh_number + 1))

	!Inicialización
	call create_molecules(nsites, mol_centers, positions, chirality, orientation, occupancy, neighbors)

	!Energia inicial
	energy = 0.0
	do jj = 1, nsites
		! Posición de la molecula "central"
		neighbor_pos(1, :, :) = positions(jj, :, :) 
		neighbor_occ(1) = occupancy(jj)
		! Se buscan las posiciones de las vecinas
		do ii = 1, neigh_number
			neighbor_pos(ii + 1, :, :) = positions(neighbors(jj, ii), :, :)
			neighbor_occ(ii + 1) = occupancy(neighbors(jj, ii))
		end do
		energy = energy + potential_energy(neighbor_pos, neighbor_occ)
		!print*, "Energia intermedia :", energy
	end do
	energy = energy / 2.0
	size_sys = dble(nsites)
	system_energy(1) = energy/size_sys
	!print*, "Energia inicial: ", system_energy(1)

	! Ciclo de Montecarlo
	! Se modificó para cambiar la temperatura poco a poco
	dtemp = (init_temperature - final_temperature)/(ntemps - 1)

	!Quiero convertir el dt lineal a uno cuadrático, para que la temperatura descienda rapido
	!al principio y lento al final, a ver si funciona para correr con side**4	
	quad_a = (init_temperature - final_temperature)/(init_temperature**2 - final_temperature**2)  
	quad_b = final_temperature - quad_a*final_temperature**2
	temp_arr(1) = init_temperature
	do ii = 1, (ntemps - 2)
		temp_arr(ii + 1) = quad_a*(init_temperature - ii*dtemp)**2 + quad_b
	end do
	temp_arr(ntemps) = final_temperature
	temp_arr(ntemps + 1) = final_temperature

	temperature = init_temperature
	lambdawl = sqrt(1/temperature) !Longitud de onda termica
	zz = exp(chempot/temperature)/(lambdawl**3) !Actividad

	!Número de ciclos que se hacen por cada temperatura
	cycles_ptemp = floor(dble(cycles)/ntemps)
	sample_counter = 1
	temp_counter = 1
	do ii = 1, cycles
		lambdawl = sqrt(1/temperature) !Longitud de onda termica
		zz = exp(chempot/temperature)/(lambdawl**3) !Actividad
		call montecarlo(nsites, nmol, mol_centers, positions, chirality, \&
			orientation, occupancy, neighbors, temperature, energy_change)

		energy = energy + energy_change
		!Muestreo
		if (mod(ii, nsample) == 0) then
			system_energy(sample_counter) = energy/size_sys
			sample_counter = sample_counter + 1
			!if (ii == cycles/10) then
			!	write(*,*) "Avance: ", 100*ii/cycles, "%"
			!end if
		end if
		!Bajar la temperatura cada cierto número de ciclos
		if (mod(ii, cycles_ptemp) == 0) then
!			print*, "Temp. intermedia: ", temperature
			temp_counter = temp_counter + 1
!			print*, temp_counter
			temperature = temp_arr(temp_counter)
			lambdawl = sqrt(1/temperature) !Longitud de onda termica
			zz = exp(chempot/temperature)/(lambdawl**3) !Actividad
		end if
	end do

	! Escritura de archivos
	! La siguiente linea es para variar el nombre del archivo conforme a lambda
	! Archivo de la configuración del sistema
	! Aqui es donde se deberia multiplicar chirality*orientation para 
	! tener en un solo numero la quiralidad y la orientación
	write(filename, '( A, "lambda", F7.5, "_final.txt" )') trim(directory), lambda
	print*, filename
	open (10, file = filename, status='UNKNOWN')
	write(10, chir_format) chirality*orientation
	close(10)
	
	!Archivo de las energias
	write(filename2, '( A, "t_energies_lambda", F7.5, ".txt" )') trim(directory), lambda
	open(20, file = filename2, status='UNKNOWN')
	write(20, *) system_energy
	close(20)

	!Deasignación de tamaño, para asignarle otra dependiendo de nps
	deallocate(neighbors)
	deallocate(neighbor_pos)
	deallocate(neighbor_occ)
end do

contains

!**********	FUNCIONES Y SUBRUTINAS		*****************

!Norma de un vector
!En el argumento iria la resta de dos vectores si es necesario
real(8) function norm(vr)
	implicit none

	real(8), dimension(2) :: vr
	norm = sqrt(vr(1)**2 + vr(2)**2)

end function norm

!Elegir un numero entero entre a y b, incluidos a y b
integer function rand_int(a, b)
	implicit none
	integer, intent(in) :: a, b ! a < b
	integer :: diff
	real(8) :: rand
	
	diff = dble(b + 1 - a)
	call RANDOM_NUMBER(rand)
	rand_int = floor(diff*rand) + a !El + 1.0 es para que tambien seleccione b

end function rand_int

!Pozo cuadrado
!El argumento es un vector
real(8) function square_well(r_ij)
	implicit none

	real(8), dimension(2) :: r_ij
	real(8) :: ans, dist
	dist = norm(r_ij)
	if (dist == 0.0) then
	    ans = 1000.0	!Es para simular infinitos, tal vez me traiga problemitas
	else if (dist > 0 .and. dist <= lambda) then
	    ans = -1.0
	else !dist > lambda
	    ans = 0.0
	end if
	square_well = ans
end function square_well
	
!Moleculas derechas
!Quiero que la función me regrese una matriz
!real(8) function dextro_molecule(square_pos) NO FUNCIONA
!real(8), dimension(3, 2) function dextro_molecule(square_pos) NO FUNCIONA
function dextro_molecule(square_pos, orientation)
!	Regresa una lista con las posiciones de los atomos de cada molecula para que sea derecha
	implicit none
	real(8), dimension(3, 2) :: pos, dextro_molecule  !Defini el tipo de la función ADENTRO, no antes del nombre
	real(8), dimension(2) :: square_pos
	integer :: orientation, ii
					!  		3
	pos(1, 1) = -l2/2.0  		!		*
	pos(1, 2) = -l1/2.0  		!   		|
	pos(2, 1) = l2/2.0  		!   		|
	pos(2, 2) = -l1/2.0  		!   		|
	pos(3, 1) = l2/2.0  		!   		|
	pos(3, 2) = l1/2.0  		!	*-------*	
					!   	1	2

	do ii = 1, 3
		if (orientation == 1) then
			pos(ii, :) = square_pos(:) + pos(ii, :)
		else if (orientation == 2) then
			! El signo menos basta para voltearla 180 grados
			pos(ii, :) = square_pos(:) - pos(ii, :)
		else
			print*, "ERROR DERECHAS"
		end if
	end do
	!La siguiente linea era lo que me faltaba :v
	dextro_molecule = pos
end function dextro_molecule

!Moleculas izquierdas
function levo_molecule(square_pos, orientation)
!	Regresa una lista con las posiciones de los atomos de cada molecula para que sea derecha
	implicit none
	real(8), dimension(3, 2) :: pos, levo_molecule  !Defini el tipo de la función ADENTRO, no antes del nombre
	real(8), dimension(2) :: square_pos
	integer :: orientation, ii
					!   	3
	pos(1, 1) = l2/2.0 		!	*
	pos(1, 2) = -l1/2.0  		!	|
	pos(2, 1) = -l2/2.0  		!	|
	pos(2, 2) = -l1/2.0  		!	|
	pos(3, 1) = -l2/2.0  		!	|
	pos(3, 2) = l1/2.0  		!	*-------*
					!	2	1
	do ii = 1, 3
		if (orientation == 1) then
			pos(ii, :) = square_pos(:) + pos(ii, :)
		else if (orientation == 2) then
			pos(ii, :) = square_pos(:) - pos(ii, :)
		else
			print*, "ERROR IZQUIERDAS"
		end if
	end do

	levo_molecule = pos
end function levo_molecule

subroutine create_molecules(nsites, mol_centers, positions, chirality, orientation, occupancy, neighbors)
	implicit none
	integer, intent(in) :: nsites
	real(8), dimension(nsites, 2), intent(out) :: mol_centers
	real(8), dimension(nsites, 3, 2), intent(out) :: positions
	integer, dimension(nsites), intent(out) :: chirality, orientation
	logical, dimension(nsites), intent(out) :: occupancy
	integer, dimension(nsites, neigh_number), intent(out) :: neighbors
	real(8) :: rand
	integer :: ii, jj, row, col, mol_index, counter
	
	! Hacer que "la mitad" de las moleculas tengan la misma quiralidad de manera aleatoria
	do ii = 1, nsites
		call RANDOM_NUMBER(rand)
		if (rand <= 0.5) then
			chirality(ii) = 1 !"D"
		else
			chirality(ii) = -1 !"L"
		end if

		! Creo que es mejor asi y no con rand_int para decir cual
		! orientación es cual
		call RANDOM_NUMBER(rand)
		if (rand < 0.5) then
			orientation(ii) = 1 !Lado corto abajo
		else
			orientation(ii) = 2 !Lado corto arriba
		end if
	end do

	!chirality = 1
	    
	! Situa las moleculas en posiciones y orientaciones aleatorias
	do jj = 0, (side - 1)
		do ii = 0, (side - 1)
			mol_index = jj*side + ii + 1
			mol_centers(mol_index, 1) = ii
			mol_centers(mol_index, 2) = jj
			if (chirality(mol_index) == -1) then
				!temp_pos = levo_molecule(mol_center(mol_index, :))
				positions(mol_index, :, :) = \&
				levo_molecule(mol_centers(mol_index, :), orientation(mol_index))
			else if (chirality(mol_index) == 1) then
			!temp_pos = dextro_molecule(mol_center(mol_index, :))
				positions(mol_index, :, :) = \&
				dextro_molecule(mol_centers(mol_index, :), orientation(mol_index))
			end if
			!Asignar vecinos
			!nps esta definido desde arriba
			counter = 1
			do col = -nps, nps
				do row = -nps, nps
					if (col == 0 .and. row == 0) then 
						cycle 	!Se salta la molecula central porque no es su propia vecina
					else
						neighbors(mol_index, counter) = modulo(jj + col, side)*side + modulo(ii + row, side) + 1 
						counter = counter + 1
					end if
				end do
			end do
		end do
	end do

	!Asignar ocupación de los sitios
	occupancy = .false.
	ii = 0
	do while (ii < nmol) !nmol es el numero (en este momento inicial) de moleculas en la red
		mol_index = rand_int(1, nsites)
		if (occupancy(mol_index) .eqv. .false.) then
			occupancy(mol_index) = .true.
			ii = ii + 1
		!	print*, "nuevamente ocupado: ", occupancy(mol_index)
		end if
		!print*,  "----------------------------"
	end do
!	print*, "Mols iniciales original: ", nmol
!	print*, "Mols iniciales contador: ", ii
		
end subroutine create_molecules

! Esta función selecciona la quiralidad contraria y elige al azar una orientación, 
! que es equivalente a elegir al azar voltear horizontal o verticalmente
! Para 4 posibles rotaciones tal vez deba cambiar el programa
subroutine change_chirality(mol_chirality, center, new_chirality, new_pos, new_orientation)
implicit none
integer, intent(in) :: mol_chirality
real(8), dimension(2), intent(in) :: center
integer, intent(out) :: new_chirality, new_orientation
real(8), dimension(3, 2), intent(out) :: new_pos
real(8) :: rand

	! Se escoge al azar si se voltea sobre
	! el lado largo o el corto
	new_orientation = rand_int(1, 2)
	
	!Si es derecha, cambiar a izquierda
	if (mol_chirality == -1) then
		new_pos = dextro_molecule(center, new_orientation)
	else if (mol_chirality == 1) then
		new_pos = levo_molecule(center, new_orientation)
	end if
	new_chirality = -mol_chirality

end subroutine change_chirality

!Esta función rota 180 grados una particula, sin cambiar su quiralidad
subroutine rotate_molecule(mol_chirality, center, old_orientation, new_pos, new_orientation)
	integer, intent(in) :: mol_chirality, old_orientation
	real(8), dimension(2), intent(in) :: center
	real(8), dimension(3, 2), intent(out) :: new_pos
	integer, intent(out) :: new_orientation
	
	!Conserva quiralidad, solo rota	
	if (mol_chirality == 1) then
		if (old_orientation == 1) then
			new_orientation = 2
			new_pos = dextro_molecule(center, new_orientation)
		else if (old_orientation == 2) then
			new_orientation = 1
			new_pos = dextro_molecule(center, new_orientation)
		end if
	else if (mol_chirality == -1) then
		if (old_orientation == 1) then
			new_orientation = 2
			new_pos = levo_molecule(center, new_orientation)
		else if (old_orientation == 2) then
			new_orientation = 1
			new_pos = levo_molecule(center, new_orientation)
		end if
	end if
end subroutine rotate_molecule

subroutine add_molecule(occupancy, mol_centers, mol_index, new_pos, new_chirality, new_orientation)
logical, dimension(nsites), intent(in) :: occupancy
real(8), dimension(nsites, 2), intent(in) :: mol_centers
integer, intent(out) :: mol_index, new_chirality, new_orientation
real(8), dimension(3, 2), intent(out) :: new_pos
integer, dimension(nsites) :: free_sites
integer :: ii, counter

real(8) :: rand

!Encontrar indices de sitios libres
counter = 0
free_sites = 0
do ii = 1, nsites
	if (occupancy(ii) .eqv. .false.) then
		counter = counter + 1
		free_sites(counter) = ii
	end if
end do

!print*, "Sitios libres"
!print*, free_sites
!print*, "Indice: ", mol_index

mol_index = free_sites(rand_int(1, counter))
new_orientation = rand_int(1, 2) !Orientacion al azar
call RANDOM_NUMBER(rand)
if (rand < 0.5) then
	new_pos = dextro_molecule(mol_centers(mol_index, :), new_orientation)
	new_chirality = 1
else
	new_pos = levo_molecule(mol_centers(mol_index, :), new_orientation)
	new_chirality = -1
end if
end subroutine add_molecule

subroutine delete_molecule(occupancy, mol_index)
logical, dimension(nsites), intent(in) :: occupancy
integer, intent(out) :: mol_index
integer, dimension(nsites) :: occupied_sites
integer :: ii, counter

!Encontrar indices de sitios ocupados
counter = 0
occupied_sites = 0
do ii = 1, nsites
	if (occupancy(ii)) then
		counter = counter + 1
		occupied_sites(counter) = ii
	end if
end do

mol_index = occupied_sites(rand_int(1, counter))
end subroutine delete_molecule

subroutine move_molecule(occupancy, old_index, new_index)
logical, dimension(nsites), intent(in) :: occupancy
integer, intent(out) :: old_index, new_index
integer, dimension(nsites) :: free_sites, occupied_sites
integer :: ii, occupied_counter, free_counter

!Encontrar indices de sitios ocupados y libres

occupied_sites = 0
free_sites = 0
occupied_counter = 0
free_counter = 0
do ii = 1, nsites
	if (occupancy(ii)) then
		occupied_counter = occupied_counter + 1
		occupied_sites(occupied_counter) = ii
	else
		free_counter = free_counter + 1
		free_sites(free_counter) = ii
	end if
end do

old_index = occupied_sites(rand_int(1, occupied_counter))
new_index = free_sites(rand_int(1, free_counter))
end subroutine move_molecule

real(8) function potential_energy(positions, occupancies)
	implicit none
	! positions incluye las posiciones de cada atomo
	! positions(1) es la molecula central

	real(8), dimension(neigh_number + 1, 3, 2) :: positions
	logical, dimension(neigh_number + 1) :: occupancies
	integer :: ii, jj, kk, aa
	real(8) :: energy, temp_dist
	real(8), dimension(2) :: dist

	energy = 0.0
	do kk = 2, (neigh_number + 1)	!Iterar sobre las particulas vecinas
		! Si el sitio central esta vacio, se sale del ciclo y energy = 0
	    	if (occupancies(1) .eqv. .false.) then
			exit
		end if
		!print*, "Vecino relativo: ", kk
		if (occupancies(kk)) then !Se hace el siguiente ciclo solo si hay vecina
			do jj = 1, 3	!	Sobre los sitios atractivos de las vecinas
				do ii = 1, 3	!Sobre las de la particula central
					! Sin condiciones periódicas
					!energy += square_well(positions[1][ii] - positions[kk][jj])
			    
				    	!Con condiciones periódicas
					do aa = 1, 2    !Coordenadas de los sitios 
						temp_dist = abs(positions(1, ii, aa) - positions(kk, jj, aa))
						if (temp_dist > side/2) then
							temp_dist = side - temp_dist
						end if
						dist(aa) = temp_dist
					end do
					!print*, "Distancia: ", dist
					energy = energy + square_well(dist)
				end do
			end do
		end if
		!print*, "Energia:", energy
	end do
	potential_energy = energy
end function potential_energy

!Criterio de Metropolis
logical function metropolis_step(exp_arg)
!Returns True if the trial configuration goes towards a region with higher probability
!or it is given the chance to explore regions with fewer probability

implicit none

real(8) :: exp_arg, w, rand
logical :: accepted

	if (exp_arg < 0.0) then
		accepted = .true. ! Se actualiza el estado del sistema
	else
		w = exp(-exp_arg)
		call RANDOM_NUMBER(rand)
		if (rand < w) then ! Energia: w
			accepted = .true. ! Tambien se actualiza el estado del sistema
	else
		accepted = .false.
		end if
	end if
	metropolis_step = accepted

end function metropolis_step

subroutine montecarlo(nsites, nmol, centers, positions, chirality, orientation, \&
			occupancy, neighbors, t_env, energy_change)
	implicit none
	! Variables de entrada y salida
	integer, intent(in) :: nsites
	integer, intent(inout) :: nmol
	real(8), dimension(nsites, 2), intent(in) :: centers
	integer, dimension(nsites, neigh_number), intent(in) :: neighbors
	real(8), dimension(nsites, 3, 2), intent(inout) :: positions
	integer, dimension(nsites), intent(inout) :: chirality, orientation
	logical, dimension(nsites), intent(inout) :: occupancy
	real(8), intent(in) :: t_env
	real(8), intent(out) :: energy_change

	!Variables internas
	integer :: ii, old_index, new_index, neighbor, trial_chir 
	real(8), dimension(neigh_number + 1, 3, 2) :: neighbor_pos, neighbor_pos2
	logical, dimension(neigh_number + 1) :: neighbor_occ, neighbor_occ2
	real(8), dimension(3, 2) :: old_pos, new_pos
	real(8) :: old_energy, trial_energy, energy_diff, rand
	integer :: new_orientation, new_chirality
	logical :: accepted
	real(8) :: p_add, p_del, p_move, p_rot, p_change_chir
	real(8), dimension(5) :: pv
	
	p_add = 0.1 ! Probabilidad de agregar una molecula
	p_del = 0.1 ! Prob. de remover 
	p_move = 0.1 ! Prob. de desplazar
	p_rot =  0.35 ! Prob. de rotar
	p_change_chir = 0.35 ! Prob. de cambiar de quiralidad

	!Las probs. anteriores tienen que sumar 1 (la prob. de intentar un cambio en el sistema)	
	pv(1) = p_add
	pv(2) = pv(1) + p_del
	pv(3) = pv(2) + p_move
	pv(4) = pv(3) + p_rot
	pv(5) = pv(4) + p_change_chir

	if (pv(5) < 0.00001) then
		print*, "Las probabilidades del metodo de Montecarlo no suman 1"
	end if
	
	! Intento de cambio: elegir entre agregar, remover, desplazar, rotar o cambiar la quiralidad de una molecula
	call RANDOM_NUMBER(rand)
	
	!Intentar agregar una molecula en un sitio vacio
	energy_change = 0.0
	if (rand <= pv(1)) then
		!Si hay sitios vacios, se puede agregar una molecula
		if (nmol < nsites) then
			!print*, "Agregar"
			call add_molecule(occupancy, centers, new_index, \&
				new_pos, new_chirality, new_orientation)
			neighbor_pos2(1, :, :) = new_pos
			neighbor_occ2(1) = .true.
			!write(*,*) "Posición: ", positions(idx, :, :) 
			do ii = 1, neigh_number
				neighbor = neighbors(new_index, ii)
				neighbor_pos2(ii + 1, :, :) = positions(neighbor, :, :)
				neighbor_occ2(ii + 1) = occupancy(neighbor)
			end do

			old_energy = 0.0 !No hay interacciones proque el sitio esta vacio
			trial_energy = potential_energy(neighbor_pos2, neighbor_occ2)
			energy_diff = trial_energy - old_energy
			accepted = metropolis_step(energy_diff/t_env - \&
						log(zz*side*side/(nmol + 1)))
			if (accepted) then
				positions(new_index, :, :) = new_pos
				chirality(new_index) = new_chirality
				orientation(new_index) = new_orientation
				occupancy(new_index) = .true.
				energy_change = energy_diff
				nmol = nmol + 1
			!else
			!	energy_change = 0.0
			end if
		end if
	
	!Intentar eliminar una molecula del sistema
	else if ((pv(1) < rand) .and. (rand <= pv(2))) then
		if (nmol > 0) then
			!print*, "Eliminar"
			call delete_molecule(occupancy, old_index)
			neighbor_pos(1, :, :) = positions(old_index, :, :)
			neighbor_occ(1) = .true.
			do ii = 1, neigh_number
				neighbor = neighbors(old_index, ii)
				neighbor_pos(ii + 1, :, :) = positions(neighbor, :, :)
				neighbor_occ(ii + 1) = occupancy(neighbor)
			end do
			old_energy = potential_energy(neighbor_pos, neighbor_occ)
			trial_energy = 0.0
			energy_diff = trial_energy - old_energy
			accepted = metropolis_step(energy_diff/t_env - \&
						log(nmol/(zz*side*side)))
			if (accepted) then
				occupancy(old_index) = .false.
				energy_change = energy_diff
				nmol = nmol - 1
			!else
			!	energy_change = 0.0
			end if
		end if

	! Intentar desplazar una particula existente a un sitio vacio
	else if ((pv(2) < rand) .and. (rand <= pv(3))) then
		!Si hay moleculas y sitios vacios, intentar mover una molecula
		!En caso contrario no se podra hacer nada
		if ((0 < nmol) .and. (nmol < nsites)) then
			!print*, "Desplazar"
			call move_molecule(occupancy, old_index, new_index)
			!print*, "Indice actual: ", old_index
			!print*, "Indice de prueba: ", new_index


! Parece que funciona excepto si es vecina de la molecula de prueba
			neighbor_pos(1, :, :) = positions(old_index, :, :)
			neighbor_occ(1) = .true.
			do ii = 1, neigh_number
				neighbor = neighbors(old_index, ii)
				neighbor_pos(ii + 1, :, :) = positions(neighbor, :, :)
				neighbor_occ(ii + 1) = occupancy(neighbor)
			end do

! No se si es el error que busco pero el hecho es que no estoy poniendo la posicion inicial en el sitio de prueba, solo estoy como "desapareciendo" la original e "insertando" la de prueba en otro lado, pero sin la config. de la inicial.
			!Este es el error que estaba comentiendo:
			!neighbor_pos2(1, :, :) = positions(new_index, :, :)
			!La particula desplazada debe tener la quiralidad y orientacion 
			! de la original pero ubicada en el nuevo sitio
			if (chirality(old_index) == 1) then
				neighbor_pos2(1, :, :) = \&
				dextro_molecule(centers(new_index, :), orientation(old_index))
			else if (chirality(old_index) == -1) then
				neighbor_pos2(1, :, :) = \&
				levo_molecule(centers(new_index, :), orientation(old_index))
			end if
			
			neighbor_occ2(1) = .true.
			do ii = 1, neigh_number
				neighbor = neighbors(new_index, ii)
				neighbor_pos2(ii + 1, :, :) = positions(neighbor, :, :)
				neighbor_occ2(ii + 1) = occupancy(neighbor)
				if (neighbor == old_index) then
					neighbor_occ2(ii + 1) = .false.
				end if
			end do

			old_energy = potential_energy(neighbor_pos, neighbor_occ)
			!print*, "Energia actual: ", old_energy
			trial_energy = potential_energy(neighbor_pos2, neighbor_occ2)
			!print*, "Energia de prueba: ", trial_energy
			energy_diff = trial_energy - old_energy
			accepted = metropolis_step((trial_energy - old_energy)/t_env)
			if (accepted) then
				!ERROR:
				!positions(new_index, :, :) = positions(old_index, :, :)
				positions(new_index, :, :) = neighbor_pos2(1, :, :)
				chirality(new_index) = chirality(old_index)
				occupancy(old_index) = .false.
				occupancy(new_index) = .true.
				energy_change = energy_diff
			!else
			!	energy_change = 0.0
			end if
		end if

	! Intentar rotar
	else if ((pv(3) < rand) .and. (rand <= pv(4))) then
		if (nmol > 0) then
			!print*, "Rotar"
			! delete_molecule solo regresa un indice de un sitio ocupado,
			! que es lo que se necesita en este momento
			call delete_molecule(occupancy, old_index)
			call rotate_molecule(chirality(old_index), centers(old_index, :),	orientation(old_index), new_pos, new_orientation)
			neighbor_pos(1, :, :) = positions(old_index, :, :)
			neighbor_occ(1) = .true.
			!write(*,*) "Posición: ", positions(idx, :, :) 
			do ii = 1, neigh_number
				neighbor = neighbors(old_index, ii)
				neighbor_pos(ii + 1, :, :) = positions(neighbor, :, :)
			end do	

			neighbor_pos2(1, :, :) = new_pos
			neighbor_pos2(2:(neigh_number + 1), :, :) = \&
						neighbor_pos2(2:(neigh_number + 1), :, :)
			neighbor_occ2 = neighbor_occ

			old_energy = potential_energy(neighbor_pos, neighbor_occ)
			trial_energy = potential_energy(neighbor_pos2, neighbor_occ2)
			energy_diff = trial_energy - old_energy
			accepted = metropolis_step((trial_energy - old_energy)/t_env)
			if (accepted) then
				positions(old_index, :, :) = new_pos
				orientation(old_index) = new_orientation
				energy_change = energy_diff
			end if
		end if
	
	!Intentar cambiar la quiralidad 
	else if (pv(4) < rand) then
		if (nmol > 0) then
			!print*, "Quiralidad"
			! delete_molecule solo regresa un indice de un sitio ocupado,
			! que es lo que se necesita en este momento
			call delete_molecule(occupancy, old_index)
			call change_chirality(chirality(old_index), \&
			centers(old_index, :), new_chirality, new_pos, new_orientation)
			neighbor_pos(1, :, :) = positions(old_index, :, :)
			neighbor_occ(1) = .true.
			!write(*,*) "Posición: ", positions(idx, :, :) 
			do ii = 1, neigh_number
				neighbor = neighbors(old_index, ii)
				neighbor_pos(ii + 1, :, :) = positions(neighbor, :, :)
			end do	
			
			neighbor_pos2(1, :, :) = new_pos
			neighbor_pos2(2:(neigh_number + 1), :, :) = \&
						neighbor_pos(2:(neigh_number + 1), :, :)
			neighbor_occ2 = neighbor_occ

			old_energy = potential_energy(neighbor_pos, neighbor_occ)
			trial_energy = potential_energy(neighbor_pos2, neighbor_occ2)
			energy_diff = trial_energy - old_energy	
			
			accepted = metropolis_step(energy_diff/t_env)
			if (accepted) then
				positions(old_index, :, :) = new_pos
				chirality(old_index) = new_chirality
				orientation(old_index) = new_orientation
				energy_change = energy_diff
			!else
			!	energy_change = 0.0
			end if
		end if
	end if

end subroutine montecarlo

end program tesis_canonico
